-- thanks to sweetman for contributing parts of the race store code
races = {}
racescount = 0
penalties = 0
currentrace = ""
cache_file = getSetting("Cache Path") .. "cascades_scores_lite.dat"

function writeScore(racename,score)
	local f = io.open(cache_file, "a+")
	f:write(racename.."|"..score.."\n")
	f:close()
	log('race score file written')
end

function findScore(racename)
	local f = io.open(cache_file, "r")
	minval = 9999
	if not f == nil then
		for line in f:lines()
		do
			line_ar = explode('|', line)
			if line_ar[1] == racename or tonumber(line_ar[2]) < minval then
				minval = tonumber(rcdt[2])
			end
		end
		log('race score file read')
	else
		log('race score file not existant')
	end
	return minval
end

function loadCheckpoints(racename) 
	for a,v in ipairs(races[racename]['positions']) do
		if a > 1 then 
		-- first checkpoint is already spawn
			if a == table.getn(races[racename]['positions']) and not loop then 
				-- put the finish thing at the end, not a normal checkpoint, only on non-loop races
				oname = races[racename]['gates'][1]
			else
				oname = races[racename]['gates'][a]
			end
			iname = 'race_'..racename..'_'..a
			--log("*creating object '"..oname.."' with name '"..iname.."'.")
			spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
		end
	end
end


	--SetupRace has several parameters:
	--  racename to show on screen (no spaces nor underscore)
	--  array of checkpoints positions
	--  array of gates names (must have the same item count as checkpoints)
	-- is loop ?
	--  Print Altitude of the next checpoint on screen ?
	-- use penalization
function setupRace(racename, positions, gates, loop, showaltitude, usepenalties)
	log("setup race")
	if not table.getn(positions) == table.getn(gates) then
		log("ERROR: gates array should have the same amount of items than checkpoints. checkpoints: "..tostring(table.getn(positions)).." gates: "..tostring(table.getn(gates)))
	else 
		if table.getn(positions) > 1 and table.getn(gates) > 1 then
			iname = 'race_'..racename..'_1'
			oname = gates[1]
			log("*creating object '"..oname.."' with name '"..iname.."'.")					
			spawnObject(oname, iname, positions[1][1],positions[1][2],positions[1][3], positions[1][4],positions[1][5],positions[1][6], raceEventHandler)
		end
	end
	-- add the race finally
	races[racename] = {}
	races[racename]['besttime'] = findScore(racename)
	races[racename]['racename'] = racename
	races[racename]['positions'] = positions
	races[racename]['isloop'] = loop
	races[racename]['startTime'] = 0
	races[racename]['lastcheckpoint'] = 1
	races[racename]['checkpointspassed'] = 1
	races[racename]['numcheckpoints'] = table.getn(positions)
	races[racename]['checkpointsloaded'] = false
	races[racename]['showaltitude'] = showaltitude
	races[racename]['gates'] = gates
	races[racename]['usepenalization'] = usepenalties
	if loop then
		races[racename]['goalnum'] = 1
	else
		races[racename]['goalnum'] = table.getn(positions)
	end
	
end

function getCheckPointFromName(instance)
	ar = explode('_', instance)
	return tonumber(ar[3])
end

function getRacenameFromName(instance)
	ar = explode('_', instance)
	return tostring(ar[2])
end

function setupArrow(position)
	--log("reposition arrow: "..position)
	txt = currentrace.."\ncheckpoint "..tostring(position).." / "..races[currentrace]['numcheckpoints']
	if position == 1 and races[currentrace]['isloop'] then
		txt = "Finish"
	end
	if position == -1 then
		-- hide it
		pointArrow(0, 0, 0, "")
	else
		data = races[currentrace]['positions'][position]
		if data then
			if races[currentrace]["showaltitude"] then
				txt = txt .. "\n\nAlt "..string.format("%.0f", races[currentrace]['positions'][position][2] * 3,2808399 ).." feets" 
			end
			pointArrow(data[1], data[2], data[3], txt)
		else
			pointArrow(0, 0, 0, "")
		end
	end
end

function raceEventHandler(instance, box)
	--log("racehandler: "..instance.." - "..box)
	checknum = getCheckPointFromName(instance)
	racename = getRacenameFromName(instance)
	if box == "penalty" and races[racename]['usepenalization'] then
		penalties = penalties + 10
	end
	
	log("just passed checkpoint number "..checknum.." of race "..racename)

	-- create an array with all required names in it
	if currentrace == "" then
		-- not racing currently
		if checknum == 1 then
			-- start of the race
			currentrace = racename
			--log(races[currentrace]['racename'])
			races[currentrace]['startTime'] = getTime()
			races[racename]['lastcheckpoint'] = 1
			races[currentrace]['checkpointspassed'] = 1
			setupArrow(2)
			startTimer()
			if not races[racename]['checkpointsloaded'] then
				loadCheckpoints(racename)
				log("raceEventHandler: created all checkpoints for "..racename)
				races[racename]['checkpointsloaded'] = true
			end
			flashMessage("Race "..racename.." started!", 4)
			--else
			-- passed some not-start checkpoint
			--flashMessage("This is checkpoint "..checknum.." of the Race "..racename.."!", 4)
		end
	elseif currentrace == racename then
		--log(checknum.."###"..races[currentrace]['numcheckpoints'].."###"..races[currentrace]['lastcheckpoint'].."###"..tostring(races[currentrace]['isloop']))
		--log("passed: "..races[currentrace]['checkpointspassed'])
		-- we hit a checkpoint from the same race!
		if checknum == races[currentrace]['lastcheckpoint'] + 1 then
			-- racing, correct checkpoint!
			if checknum == races[currentrace]['numcheckpoints'] and races[currentrace]['isloop'] then
				setupArrow(1)
			else
				setupArrow(checknum+1)
			end
			races[currentrace]['lastcheckpoint'] = checknum
			races[currentrace]['checkpointspassed'] = races[currentrace]['checkpointspassed'] + 1
			time = getTime() - races[currentrace]['startTime']
			msg = "Passed checkpoint "..checknum.." of "..(races[currentrace]['numcheckpoints']).." after "..string.format("%.3f", time).." seconds."
			flashMessage(msg, 4)
		else
			if not checknum == races[currentrace]['lastcheckpoint'] then
				-- racing, wrong checkpoint, prevent trigger of the same checkpoint twice
				flashMessage("Wrong gate! You must find and pass Checkpoint "..races[currentrace]['lastcheckpoint'] + 1, 4)
			end
		end
		if checknum == races[currentrace]['goalnum'] and races[currentrace]['checkpointspassed'] == races[racename]['numcheckpoints'] then
			-- racing, reached finish
			--time = getTime() - starttime
			msg = ""
			time = stopTimer()
			if races[racename]['usepenalization']  then
				time = time + penalties
				msg = tostring(penalties / 10) .." penalizations for touching cones !\n"
				penalties = 0
			end
			msg = msg .. "Finished! You needed "..string.format("%.3f", time).." seconds!"

			if races[currentrace]['besttime'] == 9999 or time < tonumber(races[currentrace]['besttime']) then
				races[currentrace]['besttime'] = time
				writeScore(races[currentrace]['racename'], time)
				msg = msg .. "\nNew Best Lap Time!"
			end
			msg = msg .. "\n (Best Lap: "..string.format("%.3f", races[currentrace]['besttime'])..")"
			
			
			setupArrow(-1)
			-- reset some values
			races[currentrace]['checkpointspassed'] = 0
			
			currentrace = ""
			-- flash finish message for 10 seconds
			flashMessage(msg, 10)
		end
	else
		-- flashMessage("This Checkpoint belongs to race"..racename.."!", 4)
	end

end

------------------------------------------------------------------------------------------------------------
--array of checkpoints  to use: x, y, z, rotX, rotY, rotZ
rigfreezer_points = {
{1112.550781, 60.336845, 1148.228027, 0.000000, 207.000000, -2.000000},
{1352.945313, 88.859276, 1178.591187, 0.000000, 3.000000, 0.000000},
{1342.377319, 51.248219, 813.088867, 0.000000, 39.000000, -2.000000},
{1230.766846, 50.403942, 469.130768, 0.000000, 43.000000, 7.000000},
{972.212341, 50.245770, 109.916435, 0.000000, 116.000000, 5.000000},
{799.845093, 58.079964, 88.567123, 0.000000, 48.000000, 7.000000},
{360.384796, 64.004288, 225.859650, 0.000000, 147.000000, -2.000000},
{169.733627, 69.212479, 215.398956, 0.000000, 40.000000, -7.000000}
}

-- array of gates to use for every checkpoint (must be the .odef filename of the checkpoint)
rigfreezer_gates = {
"baja_gate1",
"baja_gate1",
"baja_gate1",
"baja_gate1",
"baja_gate1",
"baja_gate1",
"baja_gate1",
"baja_gate1",
}

	--SetupRace has several parameters:
	--  racename to show on screen (no spaces nor underscore)
	--  array of checkpoints positions
	--  array of gates names (must have the same item count as checkpoints)
	-- is loop ? true or false (in lowercase, please)
	--  Print Altitude of the next checpoint on screen ? true or false (in lowercase, please)
	-- use penalization of 10 seconds if collide with the air Race checkpoint? true or false (in lowercase, please)
	--     Note: to be able to use penalization, you need a "penalty" event. Check AirRace.odef file
setupRace("Rigfreezer", rigfreezer_points, rigfreezer_gates, false, false, false)
------------------------------------------------------------------------------------------------------------


supply_route = {
{1127.342773, 61.770809, 1105.092773, 0.000000, 296.000000, -1.000000},
{88.669373, 124.595650, 725.787720, 0.000000, -323.000000, -1.000000},
{142.491119, 71.475197, 150.227310, 0.000000, -58.000000, 0.000000},
{1187.774780, 40.071972, 22.849680, 0.000000, -139.000000, 0.000000},
{1098.303711, 61.946148, 1426.746460, 0.000000, -437.000000, -1.000000},
{1237.572632, 63.871288, 842.265076, 0.000000, -453.000000, 0.000000},
{1127.342773, 61.770809, 1105.092773, 0.000000, 296.000000, -1.000000}
}

supply_route_gates ={
"baja_gate1",
"stilthouse",
"stilthouse",
"stilthouse",
"stilthouse",
"stilthouse",
"baja_gate1"
}
setupRace("supplyRoute", supply_route, supply_route_gates, false, false, false)
------------------------------------------------------------------------------------------------------------

log("Cascades_lite.terrn.lua file loaded")
	